@@ -1,6 +1,6 @@
 #![allow(unreachable_code, clippy::needless_return)]
 
-use crate::io::{decode_response, encode_request};
+use crate::io::{decode_response, encode_request, BUFFER_CAPACITY};
 use crate::model::{
     HeaderName, HeaderValue, InvalidHeader, Method, Request, Response, Status, Url,
 };
@@ -15,10 +15,20 @@ use native_tls::TlsConnector;
 use rustls::RootCertStore;
 #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
 use rustls::{ClientConfig, ClientConnection, StreamOwned};
-#[cfg(all(feature = "rustls-native-certs", not(feature = "native-tls")))]
+#[cfg(all(
+    feature = "rustls-native-certs",
+    not(feature = "rustls-platform-verifier"),
+    not(feature = "native-tls")
+))]
 use rustls_native_certs::load_native_certs;
 #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
 use rustls_pki_types::ServerName;
+#[cfg(all(
+    feature = "rustls",
+    feature = "rustls-platform-verifier",
+    not(feature = "native-tls")
+))]
+use rustls_platform_verifier::ConfigVerifierExt;
 use std::io::{BufReader, BufWriter, Error, ErrorKind, Result};
 use std::net::{SocketAddr, TcpStream};
 #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
@@ -43,10 +53,10 @@ use webpki_roots::TLS_SERVER_ROOTS;
 /// * `rustls-ring-native` to use [Rustls](https://github.com/rustls/rustls) with
 ///   the [Ring](https://github.com/briansmith/ring) cryptographic library and the host certificates.
 /// * `rustls-aws-lc-webpki` to use [Rustls](https://github.com/rustls/rustls) with
-///   the [AWS Libcrypto for Rust](https://github.com/aws/aws-lc-rs/ring) and
+///   the [AWS Libcrypto for Rust](https://github.com/aws/aws-lc-rs) and
 ///   the [Common CA Database](https://www.ccadb.org/).
 /// * `rustls-aws-lc-native` to use [Rustls](https://github.com/rustls/rustls) with
-///   the [AWS Libcrypto for Rust](https://github.com/aws/aws-lc-rs/ring) and the host certificates.
+///   the [AWS Libcrypto for Rust](https://github.com/aws/aws-lc-rs) and the host certificates.
 ///
 /// If the `flate2` feature is enabled, the client will automatically decode `gzip` and `deflate` content-encodings.
 ///
@@ -185,10 +195,11 @@ impl Client {
             "http" => {
                 let addresses = get_and_validate_socket_addresses(request.url(), 80)?;
                 let stream = self.connect(&addresses)?;
-                let stream = encode_request(request, BufWriter::new(stream))?
-                    .into_inner()
-                    .map_err(|e| e.into_error())?;
-                decode_response(BufReader::new(stream))
+                let stream =
+                    encode_request(request, BufWriter::with_capacity(BUFFER_CAPACITY, stream))?
+                        .into_inner()
+                        .map_err(|e| e.into_error())?;
+                decode_response(BufReader::with_capacity(BUFFER_CAPACITY, stream))
             }
             "https" => {
                 #[cfg(feature = "native-tls")]
@@ -204,10 +215,11 @@ impl Client {
                         })
                         .connect(host, stream)
                         .map_err(|e| Error::new(ErrorKind::Other, e))?;
-                    let stream = encode_request(request, BufWriter::new(stream))?
-                        .into_inner()
-                        .map_err(|e| e.into_error())?;
-                    return decode_response(BufReader::new(stream));
+                    let stream =
+                        encode_request(request, BufWriter::with_capacity(BUFFER_CAPACITY, stream))?
+                            .into_inner()
+                            .map_err(|e| e.into_error())?;
+                    return decode_response(BufReader::with_capacity(BUFFER_CAPACITY, stream));
                 }
                 #[cfg(all(feature = "rustls", not(feature = "native-tls")))]
                 {
@@ -225,7 +237,7 @@ impl Client {
                     let rustls_config = RUSTLS_CONFIG.get_or_init(|| {
                         #[cfg(feature = "rustls-platform-verifier")]
                         {
-                            Arc::new(rustls_platform_verifier::tls_config())
+                            Arc::new(ClientConfig::with_platform_verifier())
                         }
                         #[cfg(not(feature = "rustls-platform-verifier"))]
                         {
@@ -260,10 +272,11 @@ impl Client {
                     let connection = ClientConnection::new(Arc::clone(rustls_config), dns_name)
                         .map_err(|e| Error::new(ErrorKind::Other, e))?;
                     let stream = StreamOwned::new(connection, self.connect(&addresses)?);
-                    let stream = encode_request(request, BufWriter::new(stream))?
-                        .into_inner()
-                        .map_err(|e| e.into_error())?;
-                    return decode_response(BufReader::new(stream));
+                    let stream =
+                        encode_request(request, BufWriter::with_capacity(BUFFER_CAPACITY, stream))?
+                            .into_inner()
+                            .map_err(|e| e.into_error())?;
+                    return decode_response(BufReader::with_capacity(BUFFER_CAPACITY, stream));
                 }
                 #[cfg(not(any(feature = "native-tls", feature = "rustls")))]
                 return Err(invalid_input_error("HTTPS is not supported by the client. You should enable the `native-tls` or `rustls` feature of the `oxhttp` crate"));
@@ -283,6 +296,7 @@ impl Client {
         }?;
         stream.set_read_timeout(self.timeout)?;
         stream.set_write_timeout(self.timeout)?;
+        stream.set_nodelay(true)?;
         Ok(stream)
     }
 
