@@ -11,11 +11,6 @@
     unused_qualifications
 )]
 
-#[cfg(all(target_feature = "native-tls", target_feature = "rustls"))]
-compile_error!(
-    "Both `native-tls` and `rustls` options of oxhttp can't be enabled at the same time"
-);
-
 #[cfg(feature = "client")]
 mod client;
 mod io;
@@ -27,4 +22,4 @@ mod utils;
 #[cfg(feature = "client")]
 pub use client::Client;
 #[cfg(feature = "server")]
-pub use server::Server;
+pub use server::{ListeningServer, Server};
