@@ -1,5 +1,6 @@
 use crate::model::Headers;
-use std::cmp::min;
+#[cfg(feature = "flate2")]
+use flate2::read::{DeflateDecoder, GzDecoder};
 use std::fmt;
 use std::io::{Cursor, Error, ErrorKind, Read, Result};
 
@@ -17,6 +18,10 @@ enum BodyAlt {
         consumed_len: u64,
     },
     Chunked(Box<dyn ChunkedTransferPayload>),
+    #[cfg(feature = "flate2")]
+    DecodingDeflate(DeflateDecoder<Box<Body>>),
+    #[cfg(feature = "flate2")]
+    DecodingGzip(GzDecoder<Box<Body>>),
 }
 
 impl Body {
@@ -43,6 +48,18 @@ impl Body {
         Self(BodyAlt::Chunked(Box::new(payload)))
     }
 
+    #[cfg(feature = "flate2")]
+    pub(crate) fn decode_gzip(self) -> Self {
+        Self(BodyAlt::DecodingGzip(GzDecoder::new(Box::new(self))))
+    }
+
+    #[cfg(feature = "flate2")]
+    pub(crate) fn decode_deflate(self) -> Self {
+        Self(BodyAlt::DecodingDeflate(DeflateDecoder::new(Box::new(
+            self,
+        ))))
+    }
+
     /// The number of bytes in the body (if known).
     #[allow(clippy::len_without_is_empty)]
     #[inline]
@@ -52,6 +69,8 @@ impl Body {
             BodyAlt::SimpleBorrowed(d) => Some(d.len().try_into().unwrap()),
             BodyAlt::Sized { total_len, .. } => Some(*total_len),
             BodyAlt::Chunked(_) => None,
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingDeflate(_) | BodyAlt::DecodingGzip(_) => None,
         }
     }
 
@@ -62,12 +81,16 @@ impl Body {
         match &self.0 {
             BodyAlt::SimpleOwned(_) | BodyAlt::SimpleBorrowed(_) | BodyAlt::Sized { .. } => None,
             BodyAlt::Chunked(c) => c.trailers(),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingDeflate(c) => c.get_ref().trailers(),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingGzip(c) => c.get_ref().trailers(),
         }
     }
 
     /// Reads the full body into a vector.
     ///
-    /// WARNING: Beware of the body size!
+    /// <div class="warning">Beware of the body size!</div>
     ///
     /// ```
     /// use oxhttp::model::Body;
@@ -86,7 +109,7 @@ impl Body {
 
     /// Reads the full body into a string.
     ///
-    /// WARNING: Beware of the body size!
+    /// <div class="warning">Beware of the body size!</div>    
     ///
     /// ```
     /// use oxhttp::model::Body;
@@ -102,35 +125,59 @@ impl Body {
         self.read_to_string(&mut buf)?;
         Ok(buf)
     }
+
+    fn debug_fields<'a, 'b, 'c>(
+        &'b self,
+        s: &'c mut fmt::DebugStruct<'b, 'a>,
+    ) -> &'c mut fmt::DebugStruct<'b, 'a> {
+        match &self.0 {
+            BodyAlt::SimpleOwned(d) => s.field("content-length", &d.get_ref().len()),
+            BodyAlt::SimpleBorrowed(d) => s.field("content-length", &d.len()),
+            BodyAlt::Sized { total_len, .. } => s.field("content-length", total_len),
+            BodyAlt::Chunked(_) => s.field("transfer-encoding", &"chunked"),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingDeflate(inner) => inner
+                .get_ref()
+                .debug_fields(s.field("content-encoding", &"deflate")),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingGzip(inner) => inner
+                .get_ref()
+                .debug_fields(s.field("content-encoding", &"gzip")),
+        }
+    }
 }
 
 impl Read for Body {
     #[inline]
-    fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
+    fn read(&mut self, mut buf: &mut [u8]) -> Result<usize> {
         match &mut self.0 {
             BodyAlt::SimpleOwned(c) => c.read(buf),
             BodyAlt::SimpleBorrowed(c) => c.read(buf),
             BodyAlt::Sized {
                 content,
-                total_len,
                 consumed_len,
+                total_len,
             } => {
-                let filtered_buf_size =
-                    min(*total_len - *consumed_len, buf.len().try_into().unwrap())
-                        .try_into()
-                        .unwrap();
-                if filtered_buf_size == 0 {
-                    return Ok(0); // No need to read anything
+                let remaining_size = *total_len - *consumed_len;
+                if remaining_size < u64::try_from(buf.len()).unwrap() {
+                    buf = &mut buf[..usize::try_from(remaining_size).unwrap()];
                 }
-                let additional = content.read(&mut buf[..filtered_buf_size])?;
-                *consumed_len += u64::try_from(additional).unwrap();
-                if additional == 0 && consumed_len != total_len {
-                    // We check we do not miss some bytes
+                if buf.is_empty() {
+                    return Ok(0); // Nothing to read
+                }
+                let read = content.read(buf)?;
+                *consumed_len += u64::try_from(read).unwrap();
+                if read == 0 {
+                    // We are missing some bytes
                     return Err(Error::new(ErrorKind::ConnectionAborted, format!("The body was expected to contain {total_len} bytes but we have been able to only read {consumed_len}")));
                 }
-                Ok(additional)
+                Ok(read)
             }
             BodyAlt::Chunked(inner) => inner.read(buf),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingDeflate(inner) => inner.read(buf),
+            #[cfg(feature = "flate2")]
+            BodyAlt::DecodingGzip(inner) => inner.read(buf),
         }
     }
 }
@@ -173,17 +220,7 @@ impl From<&'static str> for Body {
 impl fmt::Debug for Body {
     #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match &self.0 {
-            BodyAlt::SimpleOwned(d) => f
-                .debug_struct("Body")
-                .field("len", &d.get_ref().len())
-                .finish(),
-            BodyAlt::SimpleBorrowed(d) => f.debug_struct("Body").field("len", &d.len()).finish(),
-            BodyAlt::Sized { total_len, .. } => {
-                f.debug_struct("Body").field("len", total_len).finish()
-            }
-            BodyAlt::Chunked(_) => f.debug_struct("Body").finish(),
-        }
+        self.debug_fields(&mut f.debug_struct("Body")).finish()
     }
 }
 
